#!/bin/csh
# Copyright  (C) 1985-2018 Intel Corporation. All rights reserved.
#
# The information and source code contained herein is the exclusive property
# of Intel Corporation and may not be disclosed, examined, or reproduced in
# whole or in part without explicit written authorization from the Company.
#
# Invoke this script with the 'source' command. For example:
#
#    source <my_install_location>/bin/compilervars.csh [ ia32 | intel64 ]

set INSTALL_DIR="<INSTALLDIR>"
set PROD_DIR="<PRODDIR>"

if ("$1" == "ia32" || "$1" == "intel64") then 
    setenv ARCH "$1_lin" 
else 
    if ($?COMPILERVARS_ARCHITECTURE) then 
        if ("$COMPILERVARS_ARCHITECTURE" == "ia32" || "$COMPILERVARS_ARCHITECTURE" == "intel64") then 
            setenv ARCH "$COMPILERVARS_ARCHITECTURE" 
        else 
            echo "ERROR: Unknown switch '$COMPILERVARS_ARCHITECTURE'. Accepted values: ia32, intel64" 
            exit 1 
        endif 
        else 
            echo "ERROR: Unknown switch '$1'. Accepted values: ia32, intel64" 
        exit 1 
    endif 
endif 

setenv ARCH_MIC "intel64_lin_mic"

if !($?PATH) then
    setenv PATH ${INSTALL_DIR}/bin/
else
    setenv PATH ${INSTALL_DIR}/bin/:${PATH}
endif

if !($?LD_LIBRARY_PATH) then
     setenv LD_LIBRARY_PATH ${PROD_DIR}/compiler/lib/${ARCH}
   else
     setenv LD_LIBRARY_PATH ${PROD_DIR}/compiler/lib/${ARCH}:${LD_LIBRARY_PATH}
endif

if !($?LIBRARY_PATH) then
       setenv LIBRARY_PATH ${PROD_DIR}/compiler/lib/${ARCH}
   else
       setenv LIBRARY_PATH ${PROD_DIR}/compiler/lib/${ARCH}:${LIBRARY_PATH}
endif

if !($?MIC_LD_LIBRARY_PATH) then
     setenv MIC_LD_LIBRARY_PATH ${PROD_DIR}/compiler/lib/${ARCH_MIC}
   else
     setenv MIC_LD_LIBRARY_PATH ${PROD_DIR}/compiler/lib/${ARCH_MIC}:${MIC_LD_LIBRARY_PATH}
endif

if !($?NLSPATH) then
       setenv NLSPATH ${PROD_DIR}/compiler/lib/${ARCH}/locale/%l_%t/%N
   else
       setenv NLSPATH ${PROD_DIR}/compiler/lib/${ARCH}/locale/%l_%t/%N:${NLSPATH}
endif

if ( "$1" == "ia32" ) then
    if ( -d "$PROD_DIR/mpirt/bin/$ARCH" ) then
        setenv PATH "${PATH}:${PROD_DIR}/mpirt/bin/${ARCH}"
    endif
    if ( -d "$PROD_DIR/mpirt/lib/$ARCH" ) then
        setenv LD_LIBRARY_PATH "${LD_LIBRARY_PATH}:${PROD_DIR}/mpirt/lib/${ARCH}"
    endif
else
    if ( -e "${PROD_DIR}/mpi/intel64/bin/mpivars.csh" ) then
        source "${PROD_DIR}/mpi/intel64/bin/mpivars.csh"
    endif
    if ( -d "${PROD_DIR}/mpi/mic/lib" ) then
        setenv MIC_LD_LIBRARY_PATH ${PROD_DIR}/mpi/mic/lib:${MIC_LD_LIBRARY_PATH};
    endif
endif
