#!/bin/bash
# Copyright 1999-2018, Intel Corporation, All Rights Reserved
# This script installs the Intel(R) redist content

# The  source code,  information and material  ("Material")  contained herein is
# owned  by  Intel  Corporation or its suppliers or licensors, and title to such
# Material  remains  with  Intel  Corporation  or  its  suppliers  or licensors.
# The  Material  contains proprietary information of  Intel or its suppliers and
# licensors.   The Material is protected by  worldwide copyright laws and treaty
# provisions. No part of the Material may be used, copied, reproduced, modified,
# published,  uploaded, posted, transmitted, distributed or disclosed in any way
# without Intel's prior express written permission. No license under any patent,
# copyright  or other intellectual property rights in the Material is granted to
# or conferred upon you, either expressly,  by implication, inducement, estoppel
# or  otherwise.  Any license  under such intellectual  property rights  must be
# express and approved by Intel in writing.
# Unless otherwise agreed by Intel in writing, you may not  remove or alter this
# notice or any other notice embedded in Materials by Intel or Intel's suppliers
# or licensors in any way.

TOUCH_SPACE()
{
    local dir_to_check=$1
    local dir_end_path='hags7823782318#@123kjhknmnzxmnz'
    local err=0
    
    if [ -d "$dir_to_check" ] ; then
        if [ -w "$dir_to_check" ] ; then
            mkdir "$dir_to_check/$dir_end_path" &> /dev/null
	    err=$?
            if [ "$err" = "0" ] ; then
                rmdir "$dir_to_check/$dir_end_path" &> /dev/null
            fi
        else
            err=1
        fi
    else
        TOUCH_SPACE "`dirname $dir_to_check`" "$dir_end_path"
        err=$?
    fi
    return $err
}

REPLACE_PATTERN() {
    local pattern="$1"
    local INSTALL_FILE="$2"
    expr="$3"

    tdir=$(echo "$expr" | sed -e"s/\// /g")
    local newexpr=""
    local dirn
    for dirn in $tdir; do
        [ "x$dirn" = "x" ] || newexpr="$newexpr\/$dirn"
    done
    cp "$INSTALL_FILE" "$INSTALL_FILE.old" 2>&1 1>/dev/null
    local lrst=$?
    if [ "x$lrst" = "x0" ]; then
	sed s/\<${pattern}\>/$newexpr/g "$INSTALL_FILE.old" 2>/dev/null 1>"$INSTALL_FILE"
	lrst=$?
	if [ "x$lrst" != "x0" ]; then
	    rm -f "$INSTALL_FILE" 2>&1 1>/dev/null
	    mv -f "$INSTALL_FILE.old" "$INSTALL_FILE" 2>&1 1>/dev/null
	else
	    rm -f "$INSTALL_FILE.old" 2>&1 1>/dev/null
	fi
    fi
}

PATCH_FILES() {
    INSTALL_SED_STR="INSTALLDIR"
    PROD_SED_STR="PRODDIR"
    
    INSTALL_LOCATION="${INSTALL_PATH}"
    PRODUCT_LOCATION="${INSTALL_PATH}/compilers_and_libraries_2019.1.144/linux"

    for FILE in $(find "${INSTALL_PATH}/bin/" 2>/dev/null | grep -e"compilervars"); do
        REPLACE_PATTERN "${INSTALL_SED_STR}" "$FILE" "${INSTALL_LOCATION}"
        REPLACE_PATTERN "${PROD_SED_STR}" "$FILE" "${PRODUCT_LOCATION}"        
    done 

    SUB_DIRS_TO_PATCH="intel64 mic"

    for subdir in $SUB_DIRS_TO_PATCH ; do
        file="${PRODUCT_LOCATION}/mpi/$subdir/bin/mpivars.sh"
        if [ -f "$file" ]; then
            cp "$file" "$file.old~"
            cat "$file.old~" | \
                sed s@I_MPI_SUBSTITUTE_INSTALLDIR@$PRODUCT_LOCATION/mpi@g > "$file.new~"
            cp "$file.new~" "$file"
            rm -f "$file.old~" "$file.new~"
        fi
    done
}

function print_help()
{
    echo "Usage: install.sh [--install-path <install location> | -i <install location> | --help]"
    echo "		-i <install location>"
    echo "		--install-path <install location>    to relocate package content"
    echo "		    -e or --eula                     to accept the terms and conditions of license agreement"
    echo "		                                     in silent (relocation) installtion mode"
    echo "		--help                               print this message and exit"
    echo ""
    echo "Copyright 2006-2018, Intel Corporation, All Rights Reserved."
}

# ---------------------------------------
#THIS_PACKAGEID=l_cprof_b_11.1.044
THIS_PACKAGEID="compilers_and_libraries_2019.1.144"
EULA="license.txt"
RELEASE_NUM=$(echo $THIS_PACKAGEID | cut -f3 -d ".")

INSTALL_PATH="$HOME"

PACKAGE="l_comp_lib_2019.1.144"

PROGRAM_DIR=$(dirname $0)
# REDIST_CONTENT="lib license.txt jp_license.txt .uninstall.sh"
REDIST_CONTENT="${THIS_PACKAGEID} bin license.txt .uninstall.sh credist fredist"
ENV_SCRIPT_SH="compilervars.sh"
ENV_SCRIPT_CSH="compilervars.csh"
THIS_PROGRAM=$(basename $0)
COMMAND_LINE=FALSE
pushd $PROGRAM_DIR > /dev/null
FILE_SRC_DIR=$(pwd)

echo "Purpose: Intel(R) Fortran Compiler 19.0 Update 1 for Linux* Redistribution Content"
echo "Package Version: $PACKAGE"
if [ "$1" != "" ]; then
    ARG1=$1
    ARG2=$2
    ARG3=$3
    if [ "$ARG1" == "--help" ] || [ "$ARG1" == "-h" ] ; then
        print_help
        exit 0
    fi
    ACCEPTED_EULA=
    if [ "$ARG1" == "--eula" ] || [ "$ARG1" == "-e" ] ; then
	ACCEPTED_EULA=TRUE
	if [ "$ARG2" == "--install-path" ] || [ "$ARG2" == "-i" ] ; then
    	    if [ "$ARG2" != "" ]; then
        	INSTALL_PATH="$ARG3"
        	COMMAND_LINE=TRUE  
	    else
    		print_help
    		exit 1
	    fi
	else
    	    print_help
    	    exit 1
	fi
    else
	if [ "$ARG1" == "--install-path" ] || [ "$ARG1" == "-i" ] ; then
    	    if [ "$ARG2" != "" ]; then
        	INSTALL_PATH="$ARG2"
        	COMMAND_LINE=TRUE  
	    else
    		print_help
    		exit 1
	    fi
	else
    	    print_help
    	    exit 1
	fi
	if [ "$ARG3" == "--eula" ] || [ "$ARG3" == "-e" ] ; then
	    ACCEPTED_EULA=TRUE
	fi
    fi
    if [ "$COMMAND_LINE" == "TRUE" ] && [ "$ACCEPTED_EULA" != "TRUE" ] ; then
	print_help
    	exit 1
    fi
fi
if [ "$COMMAND_LINE" != "TRUE" ]; then
    echo ""
    echo "--------------------------------------------------------------------------------"
    echo "Please carefully read the following license agreement.  Prior to installing the"
    echo "software you will be asked to agree to the terms and conditions of the following"
    echo "license agreement."
    echo "--------------------------------------------------------------------------------"
    echo -n "Press Enter to continue   :   "
    read CONTINUE &> /dev/null
    echo ; echo
    more "$EULA"
    echo
    ACCEPTED=FALSE
    while [ "$ACCEPTED" != "TRUE" ]
    do
        echo "Do you agree to be bound by the terms and conditions of this license agreement?"
        if [ "$VERBOSE_MODE" = "1" ]; then echo "GET_ACCEPTANCE()"; fi
        echo -n "'accept' to continue, 'reject' to return to the main menu   :   "
        read ACCEPTANCE &> /dev/null
        # eliminate white space
        ACCEPTANCE="$(echo $ACCEPTANCE | sed -e'1,$s/ *$//g')"
        ACCEPTANCE="$(echo $ACCEPTANCE | sed -e'1,$s/[[:space:]]*$//g' )"
        echo
        if echo "$ACCEPTANCE" | grep -i '^reject$' &> /dev/null ; then
            echo "The contents of this package will not be installed."
            exit 0
        elif echo "$ACCEPTANCE" | grep -i '^accept$' &> /dev/null ; then
            ACCEPTED=TRUE
        else
            ACCEPTED=FALSE
        fi
    done
    
    PROPER_INSTALL_PATH=FALSE
    while [ "$PROPER_INSTALL_PATH" == "FALSE" ]
    do
        echo -n "Install location: [$INSTALL_PATH] "
        read INSTALL_CONFIRM
        INSTALL_CONFIRM="$(echo $INSTALL_CONFIRM | sed -e'1,$s/ *$//g')"
        INSTALL_CONFIRM="$(echo $INSTALL_CONFIRM | sed -e'1,$s/[[:space:]]*$//g' )"
        if [ "$INSTALL_CONFIRM" == "" ]; then
            INSTALL_CONFIRM=$INSTALL_PATH
        fi
        if [ $(echo "$INSTALL_CONFIRM" | grep ' ' 2>/dev/null | wc -l) == "0" ]; then
            PROPER_INSTALL_PATH="TRUE"
        else
            echo "Install path cannot contain whitespaces!"
            INSTALL_CONFIRM=""
        fi
    done

    if [ "$INSTALL_CONFIRM" != "" ]; then
        INSTALL_PATH="$INSTALL_CONFIRM"
    fi

    echo "This program will install the $PACKAGE software"
    echo "in this location: $INSTALL_PATH"
    echo ""
    echo "Previous contents will be overwritten."
    echo ""
    echo -n "Continue with the installation?  (yes / no) [ default = yes ]: "
    read CONTINUE
    CONTINUE=$(echo $CONTINUE | tr [A-Z] [a-z])
    echo ""
    if [ "$CONTINUE" = "yes" ] || [ "$CONTINUE" = "y" ] || [ "$CONTINUE" = "" ]; then
        echo "Proceeding with installation program $THIS_PROGRAM ..."
    else
        echo "Aborting installation program."
        exit 1
    fi 
fi

TOUCH_SPACE $INSTALL_PATH
err=$?

if [ "x$err" != "x0" ] ; then
    echo "Cannot create directory $INSTALL_PATH due to insufficient permissions.  Exiting program."
    exit 1
fi

mkdir -p $INSTALL_PATH &>/dev/null
if [ ! -d $INSTALL_PATH ]; then
    echo "Cannot create directory $INSTALL_PATH.  Exiting program."
    exit 1
fi
pushd $INSTALL_PATH > /dev/null
for THIS_CONTENT in $REDIST_CONTENT
do
    [ -e "$FILE_SRC_DIR/$THIS_CONTENT" ] && cp -rp $FILE_SRC_DIR/$THIS_CONTENT .
done
mv ./.uninstall.sh uninstall.sh
chmod 755 uninstall.sh
popd > /dev/null
popd > /dev/null
PATCH_FILES
echo "Installation program $THIS_PROGRAM completed."
exit 0
